function gadget:GetInfo()
	return {
		name = "Map Setup",
		desc = "",
		author = "[MOL]",
		date = "",
		license = "",
		layer = 1,
		enabled = true
	}
end

VFS.Include("luarules/configs/metalSpots.lua")
VFS.Include("luarules/configs/geoVents.lua")
mex = VFS.Include("luarules/gadgets/include/mex_utilities.lua")
vent = VFS.Include("luarules/gadgets/include/vent_utilities.lua")
local map_metal = tonumber(Spring.GetMapOptions().map_metal)
local mapX = Game.mapSizeX
local mapZ = Game.mapSizeZ

if (gadgetHandler:IsSyncedCode()) then
	local PlayerGeoVents = {}
	local PlayerMetalSpots = {}
	local prevPlayerGeoVents = {}
	local prevPlayerMetalSpots = {}
	local SpotsOffset = 160

	function gadget:Initialize()
		mex.clearAll()
		mex.set(constMexSpots)
		vent.set(constVentsSpots)
	end

	local edgeOffset = 170

	function gadget:AllowStartPosition(playerID, teamID, readyState, x, y, z, rx, ry, rz)
		if x < edgeOffset or x > mapX - edgeOffset or z < edgeOffset or z > mapZ - edgeOffset then return true end

		local mexSpots = {
			{
				m = map_metal,
				x = x - SpotsOffset,
				z = z - SpotsOffset
			},
			{
				m = map_metal,
				x = x - SpotsOffset,
				z = z + SpotsOffset
			},
			{
				m = map_metal,
				x = x + SpotsOffset,
				z = z + SpotsOffset
			},
			{
				m = map_metal,
				x = x + SpotsOffset,
				z = z - SpotsOffset
			}
		}

		local geoVents = {
			{
				x = x - SpotsOffset / 1.7,
				z = z
			},
			{
				x = x + SpotsOffset / 1.7,
				z = z
			}
		}

		prevPlayerGeoVents[playerID] = PlayerGeoVents[playerID]
		prevPlayerMetalSpots[playerID] = PlayerMetalSpots[playerID]

		if prevPlayerMetalSpots[playerID] then
			mex.clear(prevPlayerMetalSpots[playerID])
		end

		if prevPlayerGeoVents[playerID] then
			vent.clear(prevPlayerGeoVents[playerID])
		end

		PlayerMetalSpots[playerID] = mexSpots
		PlayerGeoVents[playerID] = geoVents
		mex.set(mexSpots)
		vent.set(geoVents)
		_G.mexSpots = PlayerMetalSpots
		_G.geoVents = PlayerGeoVents
		SendToUnsynced("SpotsUpdate")
		mex.set(constMexSpots)

		return true
	end
else
	local displayList = 0
	local MexWidth = 130
	local MexHeight = 130
	local halfMexWidth = MexWidth / 2
	local halfMexHeight = MexHeight / 2
	local geoVentWidth = 90
	local geoVentHeight = 90
	local halfGeoVentWidth = geoVentWidth / 2
	local halfGeoVentHeight = geoVentHeight / 2
	local mexPlayersSpots = {}
	local ventsPlayersSpots = {}

	function gadget:Initialize()
		gadgetHandler:AddSyncAction("SpotsUpdate", SpotsUpdate)
		SpotsUpdate()
	end

	function gadget:ShutDown()
		gadgetHandler:RemoveSyncAction("SpotsUpdate")

		if displayList then
			gl.DeleteList(displayList)
		end
	end

	function SpotsUpdate()
		mexPlayersSpots = SYNCED.mexSpots or {}
		ventsPlayersSpots = SYNCED.geoVents or {}

		if displayList then
			gl.DeleteList(displayList)
		end

		displayList = gl.CreateList(drawPatches)
	end

	function gadget:GameStart()
	end

	function drawPatches()
		gl.MatrixMode(GL.TEXTURE)
		gl.PolygonOffset(-25, -2)
		gl.Culling(GL.BACK)
		gl.DepthTest(true)
		gl.Color(1, 1, 1, 1)
		gl.Texture(":a:luaui/images/mex.png")

		for playerID, mexSpots in pairs(mexPlayersSpots) do
			for i = 1, #mexSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(mexSpots[i].x - halfMexWidth, mexSpots[i].z - halfMexHeight, mexSpots[i].x + halfMexWidth, mexSpots[i].z + halfMexHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		if constMexSpots then
			for i = 1, #constMexSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(constMexSpots[i].x - halfMexWidth, constMexSpots[i].z - halfMexHeight, constMexSpots[i].x + halfMexWidth, constMexSpots[i].z + halfMexHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		gl.Texture(":a:luaui/images/geo.png")

		for playerID, geoVents in pairs(ventsPlayersSpots) do
			for i = 1, #geoVents do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(geoVents[i].x - halfGeoVentWidth, geoVents[i].z - halfGeoVentHeight, geoVents[i].x + halfGeoVentWidth, geoVents[i].z + halfGeoVentHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		if constVentsSpots then
			for i = 1, #constVentsSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(constVentsSpots[i].x - halfGeoVentWidth, constVentsSpots[i].z - halfGeoVentHeight, constVentsSpots[i].x + halfGeoVentWidth, constVentsSpots[i].z + halfGeoVentHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		gl.Texture(false)
		gl.DepthTest(false)
		gl.Culling(false)
		gl.PolygonOffset(false)
		gl.MatrixMode(GL.MODELVIEW)
	end

	function gadget:DrawWorldPreUnit()
		gl.CallList(displayList)
	end
end